@@ -5,17 +5,18 @@
 # termgraph.py - draw basic graphs on terminal
 # https://github.com/mkaz/termgraph
 
-from __future__ import print_function
+from __future__ import print_function, annotations
+from typing import List, Tuple, Dict
 import argparse
 import sys
 import math
 from datetime import datetime, timedelta
 from itertools import zip_longest
-from colorama import init
+from colorama import init  # type: ignore
 import os
 import re
 
-VERSION = "0.4.2"
+VERSION = "0.5.0"
 
 init()
 
@@ -31,17 +32,18 @@ AVAILABLE_COLORS = {
 }
 
 DAYS = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
+UNITS = ["", "K", "M", "B", "T"]
 DELIM = ","
 TICK = "▇"
 SM_TICK = "▏"
 
 try:
-    range = xrange
+    range = xrange  # type: ignore
 except NameError:
     pass
 
 
-def init_args():
+def init_args() -> Dict:
     """Parse and return the arguments."""
     parser = argparse.ArgumentParser(description="draw basic graphs on terminal")
     parser.add_argument(
@@ -64,6 +66,11 @@ def init_args():
     parser.add_argument(
         "--no-values", action="store_true", help="Do not print the values at end"
     )
+    parser.add_argument(
+        "--space-between",
+        action="store_true",
+        help="Print a new line after every field",
+    )
     parser.add_argument("--color", nargs="*", help="Graph bar color( s )")
     parser.add_argument("--vertical", action="store_true", help="Vertical graph")
     parser.add_argument("--stacked", action="store_true", help="Stacked bar graph")
@@ -124,23 +131,26 @@ def main():
         sys.exit()
 
     _, labels, data, colors = read_data(args)
-    if args["calendar"]:
-        calendar_heatmap(data, labels, args)
-    else:
-        chart(colors, data, args, labels)
+    try:
+        if args["calendar"]:
+            calendar_heatmap(data, labels, args)
+        else:
+            chart(colors, data, args, labels)
+    except BrokenPipeError:
+        pass
 
 
-def find_min(data):
+def find_min(data: List):
     """Return the minimum value in sublist of list."""
     return min([min(sublist) for sublist in data])
 
 
-def find_max(data):
+def find_max(data: List):
     """Return the maximum value in sublist of list."""
     return max([max(sublist) for sublist in data])
 
 
-def normalize(data, width):
+def normalize(data: List, width: int) -> List:
     """Normalize the data and return it."""
 
     # We offset by the minimum if there's a negative.
@@ -167,19 +177,42 @@ def normalize(data, width):
     return normal_data
 
 
-def find_max_label_length(labels):
+def find_max_label_length(labels: List) -> int:
     """Return the maximum length for the labels."""
-    length = 0
-    for i in range(len(labels)):
-        if len(labels[i]) > length:
-            length = len(labels[i])
-
-    return length
+    return max([len(label) for label in labels])
 
 
-def hist_rows(data, args, colors):
+def cvt_to_readable(num):
+    """Return the number in a human readable format
+
+    Eg:
+    125000 -> 125.0K
+    12550 -> 12.55K
+    19561100 -> 19.561M
+    """
+
+    if num != 0:
+        neg = num < 0
+        num = abs(num)
+
+        # Find the degree of the number like if it is in thousands or millions, etc.
+        index = math.floor(math.log(num) / math.log(1000))
+
+        # Converts the number to the human readable format and returns it.
+        newNum = round(num / (1000 ** index), 3)
+        newNum *= -1 if neg else 1
+        degree = UNITS[index]
+
+    else:
+        newNum = 0
+        degree = UNITS[0]
+
+    return (newNum, degree)
+
+
+def hist_rows(data: List, args: Dict, colors: List):
     """Prepare the Histgram graph.
-       Each row is printed through the print_row function."""
+    Each row is printed through the print_row function."""
 
     val_min = find_min(data)
     val_max = find_max(data)
@@ -238,9 +271,16 @@ def hist_rows(data, args, colors):
         print(tail)
 
 
-def horiz_rows(labels, data, normal_dat, args, colors, doprint=True):
+def horiz_rows(
+    labels: List,
+    data: List,
+    normal_dat: List,
+    args: Dict,
+    colors: List,
+    doprint: bool = True,
+):
     """Prepare the horizontal graph.
-       Each row is printed through the print_row function."""
+    Each row is printed through the print_row function."""
     val_min = find_min(data)
 
     for i in range(len(labels)):
@@ -257,6 +297,9 @@ def horiz_rows(labels, data, normal_dat, args, colors, doprint=True):
         values = data[i]
         num_blocks = normal_dat[i]
 
+        if args.get("space_between") and i != 0:
+            print()
+
         for j in range(len(values)):
             # In Multiple series graph 1st category has label at the beginning,
             # whereas the rest categories have only spaces.
@@ -264,21 +307,22 @@ def horiz_rows(labels, data, normal_dat, args, colors, doprint=True):
                 len_label = len(label)
                 label = " " * len_label
             if args.get("label_before"):
-                fmt = "{}{}"
+                fmt = "{}{}{}"
             else:
-                fmt = " {}{}"
+                fmt = " {}{}{}"
 
             if args["no_values"]:
                 tail = args["suffix"]
             else:
-                tail = fmt.format(args["format"].format(values[j]), args["suffix"])
+                val, deg = cvt_to_readable(values[j])
+                tail = fmt.format(args["format"].format(val), deg, args["suffix"])
 
             if colors:
                 color = colors[j]
             else:
                 color = None
 
-            if doprint and not args["vertical"]:
+            if not args.get("label_before") and not args.get("vertical"):
                 print(label, end="")
 
             yield (
@@ -288,19 +332,24 @@ def horiz_rows(labels, data, normal_dat, args, colors, doprint=True):
                 color,
                 label,
                 tail,
-                not doprint and not args["vertical"],
+                args.get("label_before") and not args.get("vertical"),
             )
 
-            if doprint and not args["vertical"]:
+            if not args.get("label_before") and not args.get("vertical"):
                 print(tail)
 
 
 # Prints a row of the horizontal graph.
 def print_row(
-    value, num_blocks, val_min, color, label=False, tail=False, doprint=False
+    value,
+    num_blocks: int,
+    val_min: int,
+    color: bool,
+    label: bool = False,
+    tail: bool = False,
+    doprint: bool = False,
 ):
     """A method to print a row for a horizontal graphs.
-
     i.e:
     1: ▇▇ 2
     2: ▇▇▇ 3
@@ -321,9 +370,6 @@ def print_row(
         sys.stdout.write(SM_TICK)
     else:
         if color:
-            sys.stdout.write(
-                "\033[{color}m".format(color=color)
-            )  # Start to write colorized.
             sys.stdout.write(f"\033[{color}m")  # Start to write colorized.
         for _ in range(num_blocks):
             sys.stdout.write(TICK)
@@ -335,9 +381,16 @@ def print_row(
         print()
 
 
-def stacked_graph(labels, data, normal_data, len_categories, args, colors):
+def stacked_graph(
+    labels: List,
+    data: List,
+    normal_data: List,
+    len_categories: int,
+    args: Dict,
+    colors: List,
+):
     """Prepare the horizontal stacked graph.
-       Each row is printed through the print_row function."""
+    Each row is printed through the print_row function."""
     val_min = find_min(data)
 
     for i in range(len(labels)):
@@ -347,7 +400,11 @@ def stacked_graph(labels, data, normal_data, len_categories, args, colors):
         else:
             label = "{:<{x}}: ".format(labels[i], x=find_max_label_length(labels))
 
+        if args.get("space_between") and i != 0:
+            print()
+
         print(label, end="")
+
         values = data[i]
         num_blocks = normal_data[i]
 
@@ -362,9 +419,9 @@ def stacked_graph(labels, data, normal_data, len_categories, args, colors):
 value_list, zipped_list, vertical_list, maxi = [], [], [], 0
 
 
-def vertically(value, num_blocks, val_min, color, args):
+def vertically(value, num_blocks: int, val_min: int, color: bool, args: Dict) -> List:
     """Prepare the vertical graph.
-       The whole graph is printed through the print_vertical function."""
+    The whole graph is printed through the print_vertical function."""
     global maxi, value_list
 
     value_list.append(str(value))
@@ -402,7 +459,7 @@ def vertically(value, num_blocks, val_min, color, args):
     return result_list
 
 
-def print_vertical(vertical_rows, labels, color, args):
+def print_vertical(vertical_rows: List, labels: List, color: bool, args: Dict) -> None:
     """Print the whole vertical graph."""
     if color:
         sys.stdout.write(
@@ -426,7 +483,7 @@ def print_vertical(vertical_rows, labels, color, args):
             print("  ".join(label))
 
 
-def chart(colors, data, args, labels):
+def chart(colors: List, data: List, args: Dict, labels: List) -> None:
     """Handle the normalization of data and the printing of the graph."""
     len_categories = len(data[0])
     if len_categories > 1:
@@ -460,8 +517,9 @@ def chart(colors, data, args, labels):
                         vertic = vertically(row[0], row[1], row[2], row[3], args=args)
                     else:
                         print_row(*row)
+                        print("\n")
 
-                # The above gathers data for vertical and does not pritn
+                # The above gathers data for vertical and does not print
                 # the final print happens at once here
                 if args["vertical"]:
                     print_vertical(vertic, labels, colors[i], args)
@@ -477,6 +535,7 @@ def chart(colors, data, args, labels):
 
         for row in hist_rows(data, args, colors):
             print_row(*row)
+            print()
         return
 
     # One category/Multiple series graph with same scale
@@ -484,9 +543,7 @@ def chart(colors, data, args, labels):
     if not args["stacked"]:
         normal_dat = normalize(data, args["width"])
         sys.stdout.write("\033[0m")  # no color
-        for row in horiz_rows(
-            labels, data, normal_dat, args, colors, not args.get("label_before")
-        ):
+        for row in horiz_rows(labels, data, normal_dat, args, colors):
             if not args["vertical"]:
                 print_row(*row)
             else:
@@ -504,7 +561,7 @@ def chart(colors, data, args, labels):
         print()
 
 
-def check_data(labels, data, args):
+def check_data(labels: List, data: List, args: Dict) -> List:
     """Check that all data were inserted correctly. Return the colors."""
     len_categories = len(data[0])
 
@@ -567,9 +624,9 @@ def check_data(labels, data, args):
     return colors
 
 
-def print_categories(categories, colors):
+def print_categories(categories: List, colors: List) -> None:
     """Print a tick and the category's name for each category above
-       the graph."""
+    the graph."""
     for i in range(len(categories)):
         if colors:
             sys.stdout.write(
@@ -584,17 +641,18 @@ def print_categories(categories, colors):
     print("\n\n")
 
 
-def read_data(args):
+def read_data(args: Dict) -> Tuple[List, List, List, List]:
     """Read data from a file or stdin and returns it.
 
-       Filename includes (categories), labels and data.
-       We append categories and labels to lists.
-       Data are inserted to a list of lists due to the categories.
+    Filename includes (categories), labels and data.
+    We append categories and labels to lists.
+    Data are inserted to a list of lists due to the categories.
+
+    i.e.
+    labels = ['2001', '2002', '2003', ...]
+    categories = ['boys', 'girls']
+    data = [ [20.4, 40.5], [30.7, 100.0], ...]"""
 
-       i.e.
-       labels = ['2001', '2002', '2003', ...]
-       categories = ['boys', 'girls']
-       data = [ [20.4, 40.5], [30.7, 100.0], ...]"""
     filename = args["filename"]
     stdin = filename == "-"
 
@@ -605,7 +663,9 @@ def read_data(args):
     if args["title"]:
         print("# " + args["title"] + "\n")
 
-    categories, labels, data, colors = ([] for i in range(4))
+    categories, labels, data, colors = ([] for _ in range(4))
+
+    f = None
 
     try:
         f = sys.stdin if stdin else open(filename, "r")
@@ -631,10 +691,19 @@ def read_data(args):
                             data_points.append(float(cols[i].strip()))
 
                         data.append(data_points)
+    except FileNotFoundError:
+        print(
+            ">> Error: The specified file [{fname}] does not exist.".format(
+                fname=filename
+            )
+        )
+        sys.exit()
     except IOError:
         print("An IOError has occurred!")
+        sys.exit()
     finally:
-        f.close()
+        if f is not None:
+            f.close()
 
     # Check that all data are valid. (i.e. There are no missing values.)
     colors = check_data(labels, data, args)
@@ -645,7 +714,7 @@ def read_data(args):
     return categories, labels, data, colors
 
 
-def calendar_heatmap(data, labels, args):
+def calendar_heatmap(data: Dict, labels: List, args: Dict) -> None:
     """Print a calendar heatmap."""
     if args["color"]:
         colornum = AVAILABLE_COLORS.get(args["color"][0])
